#include "ds18x20.h"
#include <string.h>

bool ds18x20_check_sensor(void)
{
    return !one_wire_reset();
}

bool ds18x20_read_RAM_single_slave(sensor_t *sensor)
{
    if(one_wire_reset())
        return false;
    one_wire_write(SKIP_ROM);
    one_wire_write(CONVERT_T);
    
    while(one_wire_read() == 0);
    
    if(one_wire_reset())
        return false;
    
    one_wire_write(SKIP_ROM);
    one_wire_write(READ_SCRATCHPAD);
    
    
    for(uint8_t i=0; i<9; i++)
    {
        sensor->scratchpad[i] = one_wire_read();
    }
    
    return true;
}

bool ds18x20_read_ROM_single_slave(sensor_t *sensor)
{
    uint8_t rom_data[8];
    
    if(one_wire_reset())
        return false;
    
    one_wire_write(READ_ROM);
    
    for(uint8_t cnt = 0; cnt < 8; cnt++)
    {
        rom_data[cnt] = one_wire_read();
    }
    
    memcpy(sensor->serial_bytes, rom_data, 8);
    
    return true;
}

uint8_t ds18x20_calculate_crc(uint8_t *data, uint8_t len)
{
    uint8_t crc = 0x00;
    while(len--)
    {
        crc ^= *data++;
        for(uint8_t i = 0; i < 8; i++)
        {
            if(crc & 0x01) crc = (crc >> 1) ^ 0x8c;
            else crc >>= 1;
        }
    }
    return crc;
}

float ds18x20_get_temp_celsius(sensor_t *sensor)
{
    float temp;
    switch((sensor->ram_reg.config & 0x60)>>5)
    {
        case RES_9_BIT:
            temp = ((float)sensor->temperature)/2.0;
            break;
        case RES_10_BIT:
            temp = ((float)sensor->temperature)/4.0;
            break;
        case RES_11_BIT:
            temp = ((float)sensor->temperature)/8.0;
            break;
        case RES_12_BIT:
            temp = ((float)sensor->temperature)/16.0;
            break;
        default:
            temp = -999;
    }
    return temp;
}


bool ds18x20_read_RAM_multi_slave(sensor_t *sensor, uint8_t *address)
{
    if(one_wire_reset())
    return false;
    one_wire_write(SKIP_ROM);
    one_wire_write(CONVERT_T);
    
    while(one_wire_read() == 0);
    
    if(one_wire_reset())
        return false;
    
    one_wire_write(MATCH_ROM);
    
    for(uint8_t cnt = 0; cnt < 8; cnt++)
    {
        one_wire_write(address[cnt]);
    }
    
    one_wire_write(READ_SCRATCHPAD);
    
    for(uint8_t i=0; i<9; i++)
    {
        sensor->scratchpad[i] = one_wire_read();
    }
    
    return true;
}