#include "one_wire.h"

static uint8_t one_wire_read_bit(void)
{
    uint8_t b;
    
    ow_SetDigitalOutput();
    ow_SetLow();
    __delay_us(2);
    ow_SetDigitalInput();
    __delay_us(13);
    b = ow_GetValue();
    __delay_us(45);
    __delay_us(10);
    return b;
}

static void one_wire_write_bit(uint8_t b)
{
    if(b)
    {
        ow_SetDigitalOutput();
        ow_SetLow();
        __delay_us(1);
        ow_SetDigitalInput();
        __delay_us(59);
    }
    else
    {
        ow_SetDigitalOutput();
        ow_SetLow();
        __delay_us(60);
        ow_SetDigitalInput();
    }
    __delay_us(10);
}

bool one_wire_reset(void)
{
    bool device_detected;
    ow_SetDigitalInput();
    while(!ow_GetValue());
    
    ow_SetDigitalOutput();
    ow_SetLow();
    
    __delay_us(480);
    
    ow_SetHigh();
    ow_SetDigitalInput();
    
    __delay_us(70);
    
    device_detected = ow_GetValue();
    
    __delay_us(410);
    
    return device_detected;
}


void one_wire_write(uint8_t write_data)
{
    for(uint8_t cnt = 0; cnt<8; cnt++)
    {
        one_wire_write_bit(write_data & 0x01);
        write_data >>= 1;
    }
}

uint8_t one_wire_read(void)
{
    uint8_t result = 0;
    for(uint8_t cnt = 0; cnt<8; cnt++)
    {
        result>>=1;
        if(one_wire_read_bit() & 0x01)
            result |= 0x80;
    }
    return result;
}