
#ifndef DS18B20_H
#define	DS18B20_H

#include <xc.h> 
#include "one_wire.h"

#define SEARCH_ROM        0xF0
#define READ_ROM          0x33
#define MATCH_ROM         0x55
#define SKIP_ROM          0xCC
#define ALARM_SEARCH      0xEC
#define CONVERT_T         0x44
#define WRITE_SCRATCHPAD  0x4E
#define READ_SCRATCHPAD   0xBE
#define COPY_SCRATCHPAD   0x48
#define RECALL_E2         0xB8
#define READ_POWER_SUPPLY 0xB4

enum type{DS18B20=0x28, DS18S20=0x10};
enum res{ RES_9_BIT=0x00, RES_10_BIT, RES_11_BIT, RES_12_BIT};

typedef struct
{
    
    union{
        uint8_t serial_bytes[8];
        struct{
            uint8_t sensor_type;
            uint8_t serial_num[6];
            uint8_t crc;
        }rom_reg;   
    };
    union{
        uint8_t scratchpad[9];
        struct{
            uint8_t temp_l;
            uint8_t temp_h;
            uint8_t tal_l;
            uint8_t tal_h;
            uint8_t config;
            uint8_t res[3];
            uint8_t crc;
        }ram_reg;
        int16_t temperature;
    };
}sensor_t;

bool ds18x20_check_sensor(void);
bool ds18x20_read_RAM_single_slave(sensor_t *sensor);
bool ds18x20_read_ROM_single_slave(sensor_t *sensor);
float ds18x20_get_temp_celsius(sensor_t *sensor);

bool ds18x20_read_RAM_multi_slave(sensor_t *sensor, uint8_t *address);


uint8_t ds18x20_calculate_crc(uint8_t *data, uint8_t len);



#endif
